// Copyright 2008 Serkan Cabi
// This file is part of Kognite.
// 
// Kognite is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Kognite is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Kognite.  If not, see <http://www.gnu.org/licenses/>.

// Includes code from Xapian examples

#include "header.h"

bool MainWindow::indexer()
{
	QMessageBox msgBox;
	msgBox.setWindowTitle(tr("Index Confirmation"));
	msgBox.setText(tr("Kognite will now index your files.\nDepending on the size of your documents, this may take a while."));
    QPushButton *indexButton = msgBox.addButton(tr("Index"), QMessageBox::ActionRole);
    msgBox.addButton(QMessageBox::Cancel);
	msgBox.exec();
	
	if (msgBox.clickedButton() == indexButton) 
	{
		int i;
		QDir *directory = new QDir(databasePath);
		QStringList files2Delete = directory->entryList();
		for(i=0;i<files2Delete.count();i++)
		{
			if(files2Delete[i].startsWith("flintlock") || files2Delete[i].startsWith("iamlock") || files2Delete[i].startsWith("position") || files2Delete[i].startsWith("postlist") || files2Delete[i].startsWith("record") || files2Delete[i].startsWith("termlist"))
				directory->remove(files2Delete[i]);
		}
		delete directory;
		
		directory = new QDir(filesPath);
		int fileCount = fileCounter(*directory);
		
		//index and show indexing progress
		
		progress.setLabelText("Indexing files...\n");
		progress.setCancelButtonText("Abort indexing");
		progress.setRange(0,fileCount);
		progress.setValue(0);
		progress.setWindowFlags(Qt::Dialog | Qt::FramelessWindowHint);
		progress.setWindowModality(Qt::WindowModal);
		progress.show();
		
		if(!dirIndexer(*directory))
		{
			statusBar()->showMessage(tr("Indexing aborted, could not index all the files"),messageTime);
			progress.close();
			progress.reset();
			delete directory;
			return false;
		}
		
		resultsList->clear();
		
		statusBar()->showMessage(tr("Indexing complete. ")+ QString::number(fileCount)+tr(" files indexed."),messageTime);
		progress.close();
		if(!initialLaunch)
		{
			refreshAll();
		}
		delete directory;
		return true;
	}
	progress.reset();
	return false;
}

int MainWindow::fileCounter(QDir directory2Count)
{
	int fileCount = directory2Count.entryList(QDir::Files).count();
	for(int i=0;i<directory2Count.entryList(QDir::NoDotAndDotDot | QDir::Dirs).count();i++)
	{
		fileCount += fileCounter(QDir(directory2Count.filePath(directory2Count.entryList(QDir::NoDotAndDotDot | QDir::Dirs)[i])));
	}
	return fileCount;
}

bool MainWindow::dirIndexer(QDir dir2Index)
{
	int fileCount = dir2Index.entryList(QDir::Files).count();
	for(int i=0;i<fileCount;i++)
	{
		progress.setValue(progress.value()+1);
		qApp->processEvents();
		if (progress.wasCanceled()) 
		{
			return false;
		}
		QString file2Index = dir2Index.entryList(QDir::Files)[i];
		if(file2Index.endsWith(".txt",Qt::CaseInsensitive) || file2Index.endsWith(".htm",Qt::CaseInsensitive) || file2Index.endsWith(".html",Qt::CaseInsensitive))
		{
			setUpdatesEnabled(false);
			progress.setLabelText("Indexing files...\n"+file2Index);
			setUpdatesEnabled(true);
			qApp->processEvents();
			if(!fileIndexer(dir2Index.filePath(file2Index)))
			{
				return false;
			}
		}
		else if(file2Index.endsWith(".pdf",Qt::CaseInsensitive))
		{
			setUpdatesEnabled(false);
			progress.setLabelText("Indexing files...\n"+file2Index);
			setUpdatesEnabled(true);
			//progress.repaint();
			qApp->processEvents();
			if(!pdfIndexer(dir2Index.filePath(file2Index)))
			{
				return false;
			}
		}
		else
		{
			Xapian::WritableDatabase db(databasePath.toStdString().c_str(), Xapian::DB_CREATE_OR_OPEN);
			Xapian::Document doc;
			Xapian::TermGenerator indexer;
			doc.set_data( dir2Index.filePath(file2Index).toStdString().c_str() );
			indexer.set_document(doc);
			indexer.index_text(file2Index.toStdString());
			db.add_document(doc);
		}
	}
	
	int dirCount = dir2Index.entryList(QDir::NoDotAndDotDot | QDir::Dirs).count();
	for(int i=0;i<dirCount;i++)
	{
		if(!dirIndexer(QDir(dir2Index.filePath(dir2Index.entryList(QDir::NoDotAndDotDot | QDir::Dirs)[i]))))
		{
			return false;
		}
	}
	return true;
}

bool MainWindow::fileIndexer(QString file2IndexPath)
{
	try
	{
		QTextEdit *indexerTextEdit = new QTextEdit(this);
		QString content;
		QString file2IndexName = QFileInfo(file2IndexPath).fileName();
		
		// Open the database for update, creating a new database if necessary.
		Xapian::WritableDatabase db(databasePath.toStdString().c_str(), Xapian::DB_CREATE_OR_OPEN);
		
		Xapian::TermGenerator indexer;
		Xapian::Stem stemmer("english");
		indexer.set_stemmer(stemmer);
		
		if (!QFile::exists(file2IndexPath))
		{
			delete indexerTextEdit;
			return false;
		}
		QFile file(file2IndexPath);
		if (!file.open(QFile::ReadOnly))
		{
			delete indexerTextEdit;
			return false;
		}
		
		QByteArray data = file.readAll();
		QTextCodec *codec = Qt::codecForHtml(data);
		QString str = codec->toUnicode(data);
		if (Qt::mightBeRichText(str)) 
		{
			indexerTextEdit->setHtml(str);
		} 
		else 
		{
			str = QString::fromLocal8Bit(data);
			indexerTextEdit->setPlainText(str);
		}
		
		content=indexerTextEdit->toPlainText();
		
		Xapian::Document doc;
		doc.set_data( file2IndexPath.toStdString().c_str() );
		
		indexer.set_document(doc);
		indexer.index_text(file2IndexName.toStdString());
		indexer.index_text(content.toStdString());
		
		// Add the document to the database
		db.add_document(doc);
		delete indexerTextEdit;
		return true;
	}		catch(const Xapian::Error &error) 
	{
		string errorMessage = "Database error in fileIndexer: "+error.get_msg();
		statusBar()->showMessage(tr(errorMessage.c_str()),messageTime);
	}
	return false;
}

void MainWindow::searcher()
{
	try
	{
		Xapian::Database db(databasePath.toStdString().c_str());
		
		// Start an enquire session
		Xapian::Enquire enquire(db);
		
		string searchString;
		searchString = (searchBox->text()).toStdString();
		for(unsigned int i=0;i<searchString.length();i++)
		{
			searchString[i] = tolower(searchString[i]);
		}
		
		// Build the query object
		vector <string> terms;
		unsigned int cutAt;
		string delim = " ";
		
		while( (cutAt = searchString.find_first_of(delim)) != searchString.npos )
		{
			if(cutAt > 0)
			{
				terms.push_back(searchString.substr(0,cutAt));
			}
			searchString = searchString.substr(cutAt+1);
		}
		
		if(searchString.length() > 0)
		{
			terms.push_back(searchString);
		}
		
		Xapian::Query query(Xapian::Query::OP_OR, terms.begin(), terms.end());
		
		// Give the query object to the enquire session
		enquire.set_query(query);
		
		// Get the top 10 results of the query
		Xapian::MSet matches = enquire.get_mset(0, 10);
		
		int itemNumber = 0;
		resultsList->clear();
		for (Xapian::MSetIterator i = matches.begin();
			 i != matches.end();
			 ++i) 
		{
			Xapian::Document doc = i.get_document();
			resultsList -> addItem(QFileInfo(doc.get_data().c_str()).fileName());
			resultsList->item(itemNumber)->setToolTip(doc.get_data().c_str());
			itemNumber++;
			//cout << "Document ID " << *i << "\t" << i.get_percent() << "% [" << doc.get_data() << "]" << endl;
    	}
		resultsList->setCurrentIndex(QModelIndex());
		searchResults = true;
		//statusBar()->showMessage(tr("Search complete"),messageTime);
	}		catch(const Xapian::Error &error) 
	{
		string errorMessage = "Database error in searcher: "+error.get_msg();
		statusBar()->showMessage(errorMessage.c_str(),messageTime);
	}
}

//idOfDocument may be a performance hog for large databases. Try to make it faster.
Xapian::docid MainWindow::idOfDocument(QString filePath)
{
	Xapian::Database db(databasePath.toStdString().c_str());
	Xapian::docid highestID = db.get_lastdocid();
	
	unsigned counter=1;
	for (counter=1;counter<=highestID;counter++)
	{
		try
		{
			if(filePath.toStdString() == db.get_document(counter).get_data())
			{
				return counter;
			}
		}
				catch(const Xapian::Error &error){}
	}
	return 0;
}

bool MainWindow::getSimilar()
{
	try
	{
		if(textEdit->document()->isModified()) fileSave();
		Xapian::Database db(databasePath.toStdString().c_str());
		Xapian::Enquire enquire(db);
		Xapian::docid idno;
		
		if(QFileInfo(fileName).fileName().isEmpty())
		{
			return false;
		}		
		
		idno = idOfDocument(fileName);
		if(idno)
		{	
			
			Xapian::RSet rSelection=Xapian::RSet();
			rSelection.add_document(idno);
			Xapian::ESet eset = enquire.get_eset(40, rSelection);
			Xapian::Query query(Xapian::Query::OP_OR, eset.begin(), eset.end());
			enquire.set_query(query);
			
			// Get the top 10 results of the query
			Xapian::MSet matches = enquire.get_mset(0, 10);
			
			// Display the results
			resultsList->clear();
			int itemNumber = 0;
			QDirModel *dirModel = new QDirModel(QStringList(),QDir::AllEntries,QDir::Name,this);
			for (Xapian::MSetIterator i = matches.begin(); i != matches.end(); ++i) 
			{
				Xapian::Document doc = i.get_document();
				
				resultsList -> addItem(QFileInfo(doc.get_data().c_str()).fileName());
				resultsList->item(itemNumber)->setToolTip(doc.get_data().c_str());
				itemNumber++;
				//resultsList->addItem(doc.get_data().c_str());
			}
			delete dirModel;
			searchResults = false;
			searchBox->setText("");
			//resultsContainer->setTitle("Similar documents:");
			//statusBar()->showMessage(tr("Similar documents listed"),messageTime);
			return true;
		}
		else
		{
			resultsList->clear();
			if(!statusBar()->currentMessage().startsWith("Database error"))
				statusBar()->showMessage(tr("Document could not be found in database. Try indexing again."),messageTime);
			return false;
		}
		
		
	}		catch(const Xapian::Error &error) 
	{
		string errorMessage = "Database error in getSimilar: "+error.get_msg();
		statusBar()->showMessage(tr(errorMessage.c_str()),messageTime);
		return false;
	}
	return false;
}

void MainWindow::renameInDB(QString oldPath, QString newPath)
{
	try
	{	
		Xapian::WritableDatabase db(databasePath.toStdString().c_str(), Xapian::DB_CREATE_OR_OPEN);
		Xapian::Document doc;
		Xapian::docid id = idOfDocument(oldPath);
		if(id)
		{
			doc = db.get_document(id);
			doc.set_data(newPath.toStdString());
			db.replace_document(id,doc);
		}
		// else
		// {
		// 	db.flush();
		// 	fileIndexer(newPath);
		// }
		
		for(int i=0; i<resultsList->count(); i++)
		{	
			if(resultsList->item(i)->toolTip() == oldPath)
			{	
				QDirModel *dirModel = new QDirModel(QStringList(),QDir::AllEntries,QDir::Name,this);
				resultsList->item(i)->setToolTip(newPath);
				resultsList->item(i)-> setText(dirModel->fileName(dirModel->index(newPath)));
				delete dirModel;
			}
		}
		
		if(fileName == oldPath) setCurrentFileName(newPath);
		
	}		catch(const Xapian::Error &error) 
	{
		string errorMessage = "Database error in renameInDB: "+error.get_msg();
		statusBar()->showMessage(errorMessage.c_str(),messageTime);
	}
}

void MainWindow::renameDirInDB(QString oldPath, QString newPath)
{
	try	
	{	
		Xapian::WritableDatabase db(databasePath.toStdString().c_str(), Xapian::DB_CREATE_OR_OPEN);
		Xapian::docid highestID = db.get_lastdocid();
		Xapian::Document doc;
		
		unsigned counter;
		QString docPath;
		for (counter=1;counter<=highestID;counter++)
		{
			try
			{
				docPath = db.get_document(counter).get_data().c_str();
				if(docPath.startsWith(oldPath))
				{
					docPath = docPath.remove(0,oldPath.size());
					docPath.prepend(newPath);
					
					doc = db.get_document(counter);
					doc.set_data(docPath.toStdString());
					db.replace_document(counter,doc);
				}
			}
			catch(const Xapian::Error &error){}
		}
		
		for(int i=0; i<resultsList->count(); i++)
		{	
			docPath = resultsList->item(i)->toolTip();
			if(docPath.startsWith(oldPath))
			{	
				docPath = docPath.remove(0,oldPath.size());
				docPath.prepend(newPath);
				
				QDirModel *dirModel = new QDirModel(QStringList(),QDir::AllEntries,QDir::Name,this);
				resultsList->item(i)->setToolTip(docPath);
				resultsList->item(i)-> setText(dirModel->fileName(dirModel->index(docPath)));
				delete dirModel;
			}
		}
		
		if(fileName.startsWith(oldPath))
		{
			fileName = fileName.remove(0,oldPath.size());
			fileName.prepend(newPath);
		}
		
		if(currentDirPath.startsWith(oldPath))
		{
			currentDirPath = currentDirPath.remove(0,oldPath.size());
			currentDirPath.prepend(newPath);
		}
		
		if(filesPath.startsWith(oldPath))
		{
			filesPath = filesPath.remove(0,oldPath.size());
			filesPath.prepend(newPath);
			updateConfigFile();
		}
		
		if(databasePath.startsWith(oldPath))
		{
			databasePath = databasePath.remove(0,oldPath.size());
			databasePath.prepend(newPath);
			updateConfigFile();
		}
		
	}	catch(const Xapian::Error &error) 
	{
		string errorMessage = "Database error in renameDirInDB: "+error.get_msg();
		statusBar()->showMessage(errorMessage.c_str(),messageTime);
	}
}

void MainWindow::removeFromDB(QString file2Remove)
{
	try
	{	
		Xapian::WritableDatabase db(databasePath.toStdString().c_str(), Xapian::DB_CREATE_OR_OPEN);
		Xapian::docid id = idOfDocument(file2Remove);
		if(id) db.delete_document(id);
	}		catch(const Xapian::Error &error) 
	{
		string errorMessage = "Database error in removeFromDB: "+error.get_msg();
		statusBar()->showMessage(errorMessage.c_str(),messageTime);
	}	
}

void MainWindow::removeDirFromDB(QString dir2Remove)
{
	try	
	{	
		Xapian::WritableDatabase db(databasePath.toStdString().c_str(), Xapian::DB_CREATE_OR_OPEN);
		Xapian::docid highestID = db.get_lastdocid();
		Xapian::Document doc;
		
		unsigned counter;
		QString docPath;
		for (counter=1;counter<=highestID;counter++)
		{
			try
			{
				docPath = db.get_document(counter).get_data().c_str();
				if(docPath.startsWith(dir2Remove))
				{
					db.delete_document(counter);
				}
			}
			catch(const Xapian::Error &error){}
		}
	}	catch(const Xapian::Error &error) 
	{
		string errorMessage = "Database error in removeDirFromDB: "+error.get_msg();
		statusBar()->showMessage(errorMessage.c_str(),messageTime);
	}	
}