// Copyright 2008 Serkan Cabi
// This file is part of Kognite.
// 
// Kognite is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Kognite is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Kognite.  If not, see <http://www.gnu.org/licenses/>.

// Includes code from QT examples

#include "header.h"

void MainWindow::setupMenuActions()
{
	//File Menu
	{
		menu = new QMenu(tr("&File"), this);
		menuBar()->addMenu(menu);
		
		a = actionNewFile = new QAction(tr("&New Plain Text"), this);
		a->setShortcut(QKeySequence::New);
		connect(a, SIGNAL(triggered()), this, SLOT(newPlainFile()));
		menu->addAction(a);
		
		a = actionNewFile = new QAction(tr("New Rich Text"), this);
		a->setShortcut(Qt::SHIFT + Qt::CTRL + Qt::Key_N);
		connect(a, SIGNAL(triggered()), this, SLOT(newRichFile()));
		menu->addAction(a);	
		
		a = actionSave = new QAction(tr("&Save"), this);
		a->setShortcut(QKeySequence::Save);
		connect(a, SIGNAL(triggered()), this, SLOT(fileSave()));
		connect(textEdit->document(), SIGNAL(modificationChanged(bool)),
				actionSave, SLOT(setEnabled(bool)));
		menu->addAction(a);
		
		a = new QAction(tr("Save &As..."), this);
		connect(a, SIGNAL(triggered()), this, SLOT(fileSaveAs()));
		a->setShortcut(Qt::SHIFT + Qt::CTRL + Qt::Key_S);
		menu->addAction(a);
		
		a = new QAction(tr("&Open Externally"), this);
		connect(a, SIGNAL(triggered()), this, SLOT(openExternally()));
		a->setShortcut(QKeySequence::Open);
		menu->addAction(a);
		
		a = actionPrint = new QAction(tr("&Print..."), this);
		a->setShortcut(QKeySequence::Print);
		connect(a, SIGNAL(triggered()), this, SLOT(filePrint()));
		menu->addAction(a);
		
		a = new QAction(tr("&Export to PDF..."), this);
		connect(a, SIGNAL(triggered()), this, SLOT(filePrintPdf()));
		a->setShortcut(Qt::SHIFT + Qt::CTRL + Qt::Key_P);
		menu->addAction(a);
		
		a = new QAction(tr("&Quit Kognite"), this);
		a->setShortcut(Qt::CTRL + Qt::Key_Q);
		connect(a, SIGNAL(triggered()), this, SLOT(close()));
		menu->addAction(a);
		
		a = new QAction(tr("About &Kognite"), this);
		connect(a, SIGNAL(triggered()), this, SLOT(about()));
		menu->addAction(a);
	}
	
	//Edit Menu
	{
		menu = new QMenu(tr("&Edit"), this);
		menuBar()->addMenu(menu);
		
		a = actionUndo = new QAction(tr("&Undo"), this);
		a->setShortcut(QKeySequence::Undo);
		connect(a, SIGNAL(triggered()), textEdit, SLOT(undo()));
		connect(textEdit->document(), SIGNAL(undoAvailable(bool)),
				actionUndo, SLOT(setEnabled(bool)));
		menu->addAction(a);
		
		a = actionRedo = new QAction(tr("&Redo"), this);
		a->setShortcut(QKeySequence::Redo);
		connect(a, SIGNAL(triggered()), textEdit, SLOT(redo()));
		connect(textEdit->document(), SIGNAL(redoAvailable(bool)),
				actionRedo, SLOT(setEnabled(bool)));
		menu->addAction(a);
		
		menu->addSeparator();
		
		a = actionCut = new QAction(tr("Cu&t"), this);
		a->setShortcut(QKeySequence::Cut);
		connect(a, SIGNAL(triggered()), textEdit, SLOT(cut()));
		connect(textEdit, SIGNAL(copyAvailable(bool)),
				actionCut, SLOT(setEnabled(bool)));
		menu->addAction(a);
		
		a = actionCopy = new QAction(tr("&Copy"), this);
		a->setShortcut(QKeySequence::Copy);
		connect(a, SIGNAL(triggered()), textEdit, SLOT(copy()));
		connect(textEdit, SIGNAL(copyAvailable(bool)),
				actionCopy, SLOT(setEnabled(bool)));
		menu->addAction(a);
		
		a = actionPaste = new QAction(tr("&Paste"), this);
		a->setShortcut(QKeySequence::Paste);
		connect(a, SIGNAL(triggered()), textEdit, SLOT(paste()));
		//actionPaste->setEnabled(!QApplication::clipboard()->text().isEmpty());
		menu->addAction(a);
		
		a = actionCrop = new QAction(tr("C&rop"), this);
		a->setShortcut(Qt::SHIFT + Qt::CTRL + Qt::Key_C);
		connect(a, SIGNAL(triggered()), this, SLOT(crop()));
		connect(textEdit, SIGNAL(copyAvailable(bool)),
				actionCrop, SLOT(setEnabled(bool)));
		menu->addAction(a);
		
		a = actionSelectAll = new QAction(tr("Select &All"), this);
		a->setShortcut(QKeySequence::SelectAll);
		connect(a, SIGNAL(triggered()), textEdit, SLOT(selectAll()));
		menu->addAction(a);
		
		menu->addSeparator();
		
		a = actionFind = new QAction(tr("&Find..."), this);
		a->setShortcut(QKeySequence::Find);
		connect(a, SIGNAL(triggered()), this, SLOT(showFindBox()));
		menu->addAction(a);
		
		a = actionFindNext = new QAction(tr("Find Next..."), this);
		a->setShortcut(QKeySequence::FindNext);
		connect(a, SIGNAL(triggered()), this, SLOT(findNext()));
		menu->addAction(a);
		
		a = actionFindPrev = new QAction(tr("Find Previous..."), this);
		a->setShortcut(QKeySequence::FindPrevious);
		connect(a, SIGNAL(triggered()), this, SLOT(findPrev()));
		menu->addAction(a);
		
		a = actionFindAndReplace = new QAction(tr("Find and Replace..."), this);
#ifdef Q_WS_MAC
		a->setShortcut(Qt::SHIFT + Qt::CTRL + Qt::Key_F);
#else
		a->setShortcut(QKeySequence::Replace);
#endif	
		connect(a, SIGNAL(triggered()), this, SLOT(showFindAndReplace()));
		
		menu->addAction(a);
	}
	
	//View Menu
	{
		menu = new QMenu(tr("&View"), this);
		menuBar()->addMenu(menu);
		
		actionZoomIn = new QAction(tr("Zoom &In"), this);
		actionZoomIn->setShortcut(Qt::CTRL + Qt::Key_Plus);
		connect(actionZoomIn, SIGNAL(triggered()), this, SLOT(pdfZoomIn()));
		menu->addAction(actionZoomIn);
		
		actionZoomOut = new QAction(tr("Zoom &Out"), this);
		actionZoomOut->setShortcut(Qt::CTRL + Qt::Key_Minus);
		connect(actionZoomOut, SIGNAL(triggered()), this, SLOT(pdfZoomOut()));
		menu->addAction(actionZoomOut);
		
		menu->addSeparator();
		
		actionToolBar = new QAction(tr("&Toolbar"), this);
		actionToolBar->setShortcut(Qt::ALT + Qt::CTRL + Qt::Key_T);
		connect(actionToolBar, SIGNAL(triggered()), this, SLOT(toggleToolBar()));
		menu->addAction(actionToolBar);
		actionToolBar->setCheckable(true);
		
		actionFullScreen = new QAction(tr("&Full Screen"), this);
		connect(actionFullScreen, SIGNAL(triggered()), this, SLOT(toggleFullScreen()));
		actionFullScreen->setShortcut(Qt::ALT + Qt::CTRL + Qt::Key_F);
		actionFullScreen->setCheckable(true);
		menu->addAction(actionFullScreen);
		
		actionMiddleView = new QAction(tr("Flip Layout"), this);
		connect(actionMiddleView, SIGNAL(triggered()), this, SLOT(toggleMiddleView()));
		menu->addAction(actionMiddleView);
		
		a = new QAction(tr("&Refresh Files"), this);
		connect(a, SIGNAL(triggered()), this, SLOT(refreshAll()));
		menu->addAction(a);
		
		a = new QAction(tr("Re&set View"), this);
		connect(a, SIGNAL(triggered()), this, SLOT(resetView()));
		menu->addAction(a);
	}
	
	//Format Menu
	{
		menu = new QMenu(tr("F&ormat"), this);
		menuBar()->addMenu(menu);
		
		actionTextBold = new QAction(tr("&Bold"), this);
		actionTextBold->setShortcut(Qt::CTRL + Qt::Key_B);
		connect(actionTextBold, SIGNAL(triggered()), this, SLOT(textBold()));
		menu->addAction(actionTextBold);
		actionTextBold->setCheckable(true);
		
		actionTextItalic = new QAction(tr("&Italic"), this);
		actionTextItalic->setShortcut(Qt::CTRL + Qt::Key_I);
		connect(actionTextItalic, SIGNAL(triggered()), this, SLOT(textItalic()));
		menu->addAction(actionTextItalic);
		actionTextItalic->setCheckable(true);
		
		actionTextUnderline = new QAction(tr("&Underline"), this);
		actionTextUnderline->setShortcut(Qt::CTRL + Qt::Key_U);
		connect(actionTextUnderline, SIGNAL(triggered()), this, SLOT(textUnderline()));
		menu->addAction(actionTextUnderline);
		actionTextUnderline->setCheckable(true);
		
		menu->addSeparator();
		
		QActionGroup *grp = new QActionGroup(this);
		connect(grp, SIGNAL(triggered(QAction *)), this, SLOT(textAlign(QAction *)));
		
		actionAlignLeft = new QAction(tr("&Left"), grp);
		actionAlignLeft->setShortcut(Qt::CTRL + Qt::Key_L);
		actionAlignLeft->setCheckable(true);
		actionAlignCenter = new QAction(tr("C&enter"), grp);
		actionAlignCenter->setShortcut(Qt::CTRL + Qt::Key_E);
		actionAlignCenter->setCheckable(true);
		actionAlignRight = new QAction(tr("&Right"), grp);
		actionAlignRight->setShortcut(Qt::CTRL + Qt::Key_R);
		actionAlignRight->setCheckable(true);
		actionAlignJustify = new QAction(tr("&Justify"), grp);
		actionAlignJustify->setShortcut(Qt::CTRL + Qt::Key_J);
		actionAlignJustify->setCheckable(true);
		
		menu->addActions(grp->actions());
		
		menu->addSeparator();
		
		actionTextColor = new QAction(tr("&Color..."), this);
		connect(actionTextColor, SIGNAL(triggered()), this, SLOT(textColor()));
		actionTextColor->setShortcut(Qt::ALT + Qt::CTRL + Qt::Key_C);
		menu->addAction(actionTextColor);
	}
	
	//Database Menu
	{
		menu = new QMenu(tr("&Database"), this);
		menuBar()->addMenu(menu);
		
		a = new QAction(tr("&Index Files"), this);
		connect(a, SIGNAL(triggered()), this, SLOT(indexer()));
		menu->addAction(a);
		
		a = new QAction(tr("Set &Files Directory..."), this);
		connect(a, SIGNAL(triggered()), this, SLOT(chooseFilesPath()));
		menu->addAction(a);
		
		a = new QAction(tr("Set &Database Directory..."), this);
		connect(a, SIGNAL(triggered()), this, SLOT(chooseDbPath()));
		menu->addAction(a);
	}
	
}

void MainWindow::newPlainFile()
{
    if (maybeSave()) {
        textEdit->clear();
		textEdit->setCurrentCharFormat(QTextCharFormat());
		
		QDir dirObject;
		QString newFileName = currentDirPath+"Untitled";
		
		while(dirObject.exists(newFileName+".txt"))
		{
			newFileName.append("_");
		}
		newFileName.append(".txt");
		QFile file(newFileName);
		file.open(QFile::WriteOnly);
		file.close();
		refreshAll();
		setCurrentFileName(newFileName);
		fileIndexer(fileName);
		QModelIndex index = filesModel->index(newFileName);
		forcedExtension1 = ".txt";
		setRichTextActions(false);
		resultsList->selectionModel()->reset();
		renameFileStart(index);
    }
}

void MainWindow::newRichFile()
{
	if (maybeSave()) {
        textEdit->clear();
		textEdit->setCurrentCharFormat(QTextCharFormat());
		
		QDir dirObject;
		QString newFileName = currentDirPath+"Untitled";
		
		while(dirObject.exists(newFileName+".html"))
		{
			newFileName.append("_");
		}
		newFileName.append(".html");
		QFile file(newFileName);
		file.open(QFile::WriteOnly);
		file.close();
		refreshAll();
		setCurrentFileName(newFileName);
		fileIndexer(fileName);
		QModelIndex index = filesModel->index(newFileName);
		renameFileStart(index);
		setRichTextActions(true);
		forcedExtension1 = ".html";
		forcedExtension2 = ".htm";
		resultsList->selectionModel()->reset();
    }
}

bool MainWindow::fileSave()
{
    if (fileName.isEmpty())
        return fileSaveAs();
	
    QFile file(fileName);
    if (!file.open(QFile::WriteOnly))
        return false;
    QTextStream ts(&file);
	if(fileName.endsWith(".htm",Qt::CaseInsensitive) || fileName.endsWith(".html",Qt::CaseInsensitive))
	{
		ts.setCodec(QTextCodec::codecForName("UTF-8"));
		ts << textEdit->document()->toHtml("UTF-8");
	}
	else 
	{
		ts << textEdit->document()->toPlainText();
	}
	file.close();
	textEdit->document()->setModified(false);
	refreshAll();
	removeFromDB(fileName);
	fileIndexer(fileName);
	return true;
}

bool MainWindow::fileSaveAs()
{
	QString *fileType = new QString();
	if(textEdit->acceptRichText())
		*fileType = "Rich Text (*.html *.htm)";
	else
		*fileType = "Plain Text (*.txt)";
	
	QString fn = QFileDialog::getSaveFileName(this, tr("Save as..."),
											  currentDirPath+"untitled file", tr("Plain Text (*.txt);;Rich Text (*.html *.htm);;All Files (*)"),fileType);
	
	delete fileType;
    if (fn.isEmpty())
        return false;
    setCurrentFileName(fn);
    return fileSave();
}

bool MainWindow::maybeSave()
{
    if (!textEdit->document()->isModified())
        return true;
    if (fileName.startsWith(QLatin1String(":/")))
        return true;
    QMessageBox::StandardButton ret;
    ret = QMessageBox::warning(this, tr("Saving Reminder"),
                               tr("The document has been modified.\n"
                                  "Do you want to save your changes?"),
                               QMessageBox::Save | QMessageBox::Discard
                               | QMessageBox::Cancel);
    if (ret == QMessageBox::Save)
        return fileSave();
    else if (ret == QMessageBox::Cancel)
        return false;
    return true;
}

void MainWindow::filePrint()
{
#ifndef QT_NO_PRINTER
    QPrinter printer(QPrinter::HighResolution);
    printer.setFullPage(true);
    QPrintDialog *dlg = new QPrintDialog(&printer, this);
    if (textEdit->textCursor().hasSelection())
        dlg->addEnabledOption(QAbstractPrintDialog::PrintSelection);
    dlg->setWindowTitle(tr("Print Document"));
    if (dlg->exec() == QDialog::Accepted) {
        textEdit->print(&printer);
    }
    delete dlg;
#endif
}

void MainWindow::filePrintPdf()
{
#ifndef QT_NO_PRINTER
    QString filePath = QFileDialog::getSaveFileName(this, "Export PDF",
                                                    QString(), "*.pdf");
    if (!filePath.isEmpty()) {
        if (QFileInfo(filePath).suffix().isEmpty())
            filePath.append(".pdf");
        QPrinter printer(QPrinter::HighResolution);
        printer.setOutputFormat(QPrinter::PdfFormat);
        printer.setOutputFileName(filePath);
        textEdit->document()->print(&printer);
    }
#endif
}

void MainWindow::refreshAll()
{
	refreshFilesTree();
	refreshDirsTree();
}

void MainWindow::toggleFullScreen()
{
	inFullScreen = !inFullScreen;
	
	if(inFullScreen)
	{	
		winGeo = geometry();
		middleSizes = middleFrame->sizes();
		horizontalSizes = central->sizes();
		
		setUpdatesEnabled(false);
		hide();
		setUnifiedTitleAndToolBarOnMac(false);
		textEdit->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
		textEdit->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
		tb->hide();
		//menuBar()->hide();
		DocumentViewerLayout->setColumnStretch(0,1);
		DocumentViewerLayout->setColumnStretch(1,3);
		DocumentViewerLayout->setColumnStretch(2,1);
		middleFrame->setContentsMargins(0,0,0,0);
		leftFrame->hide();
		filesTree->hide();
		rightFrame->hide();
	 	statusBar()->clearMessage();
		findFrame->hide();
		actionToolBar->setEnabled(false);
		actionMiddleView->setEnabled(false);
		QPalette p = palette();
		p.setColor(QPalette::Window, Qt::black);	//background
		p.setColor(QPalette::WindowText, Qt::green);
		if(!textEdit->acceptRichText())
		{
			p.setColor(QPalette::Base, Qt::black);		//page
			p.setColor(QPalette::Text, Qt::green);		//text
		}
		textEdit->setFrameStyle(QFrame::NoFrame);	//no frame
		setPalette(p);
		//setForegroundRole(QPalette::Text);
		p.setColor(QPalette::Inactive, QPalette::Highlight,
				   p.color(QPalette::Active, QPalette::Highlight));
		p.setColor(QPalette::Inactive, QPalette::HighlightedText,
				   p.color(QPalette::Active, QPalette::HighlightedText));
		textEdit->setPalette(p);
		p.setColor(QPalette::WindowText, Qt::white);
		checkWholeWords->setPalette(p);
		checkCase->setPalette(p);
		findBox->setPalette(QApplication::palette());
		replaceBox->setPalette(QApplication::palette());
		cantOpenWidget->setPalette(QApplication::palette());
		
		if(fileName.endsWith(".pdf"))
		{	
			DocumentViewerLayout->setColumnStretch(0,0);
			DocumentViewerLayout->setColumnStretch(1,1);
			DocumentViewerLayout->setColumnStretch(2,0);
			
			DocumentViewerLayout->setColumnMinimumWidth(1,pdfWidth+30);
			pdfViewer->setFrameStyle(QFrame::NoFrame);
			pdfViewer->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
			pdfViewer->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
		}
		show();
		setUpdatesEnabled(true);
		showFullScreen();
	}
	else
	{		
		setUpdatesEnabled(false);
		hide();
		textEdit->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
		textEdit->setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);
		setPalette(QApplication::palette());
		checkWholeWords->setPalette(QApplication::palette());
		checkCase->setPalette(QApplication::palette());
		QPalette p = palette();
		p.setColor(QPalette::Inactive, QPalette::Highlight,
				   p.color(QPalette::Active, QPalette::Highlight));
		p.setColor(QPalette::Inactive, QPalette::HighlightedText,
				   p.color(QPalette::Active, QPalette::HighlightedText));
		textEdit->setPalette(p);
		textEdit->setFrameStyle(defaultTextFrameStyle);
		setWindowState(Qt::WindowNoState);
		DocumentViewerLayout->setColumnStretch(0,0);
		DocumentViewerLayout->setColumnStretch(1,1);
		DocumentViewerLayout->setColumnStretch(2,0);
		middleFrame->setContentsMargins(0,5,0,0);
		leftFrame->show();
		filesTree->show();
		rightFrame->show();
		statusBar()->show();
		//menuBar()->show();
		middleFrame->setSizes(middleSizes);
		central->setSizes(horizontalSizes);
		if (actionToolBar->isChecked()) 
		{
			toggleToolBar();
			actionToolBar->setChecked(true); //need this, don't know why?!?
		}
		actionToolBar->setEnabled(true);
		actionMiddleView->setEnabled(true);
		if(fileName.endsWith(".pdf"))
		{
			DocumentViewerLayout->setColumnMinimumWidth(1,0);
			pdfViewer->setFrameStyle(defaultPdfFrameStyle);
			pdfViewer->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
			pdfViewer->setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);
		}
		show();
		setGeometry(winGeo);
		setUpdatesEnabled(true);
		activateWindow();
		textEdit->setFocus();
		//setGeometry(winGeo);
	}
	actionFullScreen->setChecked(inFullScreen);
}

void MainWindow::toggleToolBar()
{
	QPoint tempPos = pos();
    QSize tempSize = size();
	setUnifiedTitleAndToolBarOnMac(!unifiedTitleAndToolBarOnMac());
	tb->setVisible(!tb->isVisible());
	actionToolBar->setChecked(tb->isVisible());
	if(!inFullScreen)
	{
		tempSize.setHeight(tempSize.height()+tempPos.y()-pos().y());
		resize(tempSize);
	}
}

void MainWindow::toggleMiddleView()
{
	int totalSize;
	QList<int> sizeList;
	
	if(middleFrame->orientation() == Qt::Vertical)
	{
		nameColumnWidth = filesTree->columnWidth(0);
		filesTree->hideColumn(1);
		filesTree->hideColumn(2);
		filesTree->hideColumn(3);
		middleFrame->setOrientation(Qt::Horizontal);
		totalSize = middleFrame->sizes()[0] + middleFrame->sizes()[1];
		sizeList = QList<int>();
		sizeList << 1*totalSize/3 << 2*totalSize/3;
		middleFrame->setSizes(sizeList);
		filesTree->setColumnWidth(0, middleFrame->sizes()[0]-50);
	}
	else
	{	
		filesTree->showColumn(1);
		filesTree->showColumn(2);
		filesTree->showColumn(3);
		middleFrame->setOrientation(Qt::Vertical);
		totalSize = middleFrame->sizes()[0] + middleFrame->sizes()[1];
		sizeList = QList<int>();
		sizeList << 1*totalSize/3 << 2*totalSize/3;
		middleFrame->setSizes(sizeList);
		if (nameColumnWidth) filesTree->setColumnWidth(0, nameColumnWidth);
		filesTree->resizeColumnToContents(1);
		filesTree->resizeColumnToContents(2);
		filesTree->resizeColumnToContents(3);
	}
	
}

void MainWindow::resetView()
{
	if(inFullScreen) toggleFullScreen();
	if(!tb->isVisible()) toggleToolBar();
	if(middleFrame->orientation() == Qt::Horizontal) toggleMiddleView();
	setWindowState(Qt::WindowMaximized);
	
	setUpdatesEnabled(false);
	int totalSize = central->sizes()[0] + central->sizes()[1] + central->sizes()[2];
	QList<int> sizeList;
    sizeList << 2*totalSize/11 << 7*totalSize/11 << 2*totalSize/11;	
	central->setSizes(sizeList);
	
	totalSize = middleFrame->sizes()[0] + middleFrame->sizes()[1];
	sizeList = QList<int>();
	sizeList << 1*totalSize/3 << 2*totalSize/3;
	middleFrame->setSizes(sizeList);
	
	filesTree->setColumnWidth(0,250);
	filesTree->resizeColumnToContents(1);
	filesTree->resizeColumnToContents(2);
	filesTree->resizeColumnToContents(3);
	setUpdatesEnabled(true);
}

void MainWindow::crop()
{
	textEdit->copy();
	textEdit->selectAll();
	textEdit->paste();
}

void MainWindow::about()
{
    QMessageBox::about(this, tr("About Kognite"), tr("Kognite\n" 
													 "Ignites your cognition.\n\n"
													 "This program is a free, open-source, GPL licensed software.\n"
													 "Coded by Serkan Cabi."));
}

void MainWindow::textBold()
{
    QTextCharFormat fmt;
    fmt.setFontWeight(actionTextBold->isChecked() ? QFont::Bold : QFont::Normal);
    mergeFormatOnWordOrSelection(fmt);
}

void MainWindow::textUnderline()
{
    QTextCharFormat fmt;
    fmt.setFontUnderline(actionTextUnderline->isChecked());
    mergeFormatOnWordOrSelection(fmt);
}

void MainWindow::textItalic()
{
    QTextCharFormat fmt;
    fmt.setFontItalic(actionTextItalic->isChecked());
    mergeFormatOnWordOrSelection(fmt);
}

void MainWindow::textColor()
{
    QColor col = QColorDialog::getColor(textEdit->textColor(), this);
    if (!col.isValid())
        return;
    QTextCharFormat fmt;
    fmt.setForeground(col);
    mergeFormatOnWordOrSelection(fmt);
}

void MainWindow::textAlign(QAction *a)
{
    if (a == actionAlignLeft)
        textEdit->setAlignment(Qt::AlignLeft);
    else if (a == actionAlignCenter)
        textEdit->setAlignment(Qt::AlignHCenter);
    else if (a == actionAlignRight)
        textEdit->setAlignment(Qt::AlignRight);
    else if (a == actionAlignJustify)
        textEdit->setAlignment(Qt::AlignJustify);
}

void MainWindow::mergeFormatOnWordOrSelection(const QTextCharFormat &format)
{
    QTextCursor cursor = textEdit->textCursor();
    if (!cursor.hasSelection())
        cursor.select(QTextCursor::WordUnderCursor);
    cursor.mergeCharFormat(format);
    textEdit->mergeCurrentCharFormat(format);
}

void MainWindow::chooseFilesPath(bool shouldIndex)
{
	//If you change check that it works without a config file while booting
	do
	{
		QString fdn = QFileDialog::getExistingDirectory(this,tr("Choose Files Directory"),QString(),QFileDialog::ShowDirsOnly);
		if (!fdn.isEmpty())
		{	
			if (fdn.at(fdn.length()-1)!='/') fdn.append("/"); //Don't forget the / in the directory name
			filesPath = fdn;
			currentDirPath = filesPath;
			if(!initialLaunch)
			{
				for (int i=0;i<dirsModel->rowCount(parentDirIndex);i++)
				{	
					dirsTree->setRowHidden(i,parentDirIndex,false);
				}	
				statusBar()->showMessage(tr("Files Directory is set to ") + fdn,messageTime);
				prepareDirsTree();
				refreshFilesTree();
				updateConfigFile();
			}
		}
	}
	while(!QDir(filesPath).exists() || !filesPath.endsWith('/'));
	if (shouldIndex) indexer();
}

void MainWindow::chooseDbPath(bool shouldIndex)
{
	//If you change check that it works without a config file while booting
	QString fdn;
	do
	{
		fdn = QFileDialog::getExistingDirectory(this,tr("Choose Database Directory"),QString(),QFileDialog::ShowDirsOnly);
		if (!fdn.isEmpty())
		{	
			if (fdn.at(fdn.length()-1)!='/') fdn.append("/");
			
			if (fdn.startsWith(filesPath))
			{
				QMessageBox::warning(this, tr("Invalid directory"), tr("Index files can not be inside your files directory.\n"), QMessageBox::Ok);	
			}
			else
			{
				databasePath = fdn;
				updateConfigFile();
				QString message = tr("Database Directory is set to ");
				message.append(fdn);
				statusBar()->showMessage(message,messageTime);
			}
		}
	}
	while(!QDir(databasePath).exists() || !databasePath.endsWith('/') || fdn.startsWith(filesPath));
	if (shouldIndex) indexer();
}

void MainWindow::setRichTextActions(bool isRich)
{
	textEdit->setAcceptRichText(isRich);
	actionTextBold->setEnabled(isRich);
	actionTextItalic->setEnabled(isRich);
	actionTextUnderline->setEnabled(isRich);
	actionAlignLeft->setEnabled(isRich);
	actionAlignCenter->setEnabled(isRich);
	actionAlignRight->setEnabled(isRich);
	actionAlignJustify->setEnabled(isRich);
	actionTextColor->setEnabled(isRich);
	if(isRich) textEdit->setAutoFormatting(QTextEdit::AutoAll);
	else textEdit->setAutoFormatting(QTextEdit::AutoNone);
	
}

void MainWindow::currentCharFormatChanged(const QTextCharFormat &format)
{
    fontChanged(format.font());
}

void MainWindow::cursorPositionChanged()
{
    alignmentChanged(textEdit->alignment());
}

void MainWindow::fontChanged(const QFont &f)
{
    actionTextBold->setChecked(textEdit->acceptRichText() && f.bold());
    actionTextItalic->setChecked(textEdit->acceptRichText() && f.italic());
    actionTextUnderline->setChecked(textEdit->acceptRichText() && f.underline());
}

void MainWindow::alignmentChanged(Qt::Alignment a)
{
    if (a & Qt::AlignLeft) {
        actionAlignLeft->setChecked(textEdit->acceptRichText());
    } else if (a & Qt::AlignHCenter) {
        actionAlignCenter->setChecked(textEdit->acceptRichText());
    } else if (a & Qt::AlignRight) {
        actionAlignRight->setChecked(textEdit->acceptRichText());
    } else if (a & Qt::AlignJustify) {
        actionAlignJustify->setChecked(textEdit->acceptRichText());
    }
}

// void MainWindow::fileOpen()
// {
//     QString fn = QFileDialog::getOpenFileName(this, tr("Open File..."),
// 		QString(), tr("All Known Files(*.txt *.html *.htm);;Plain Text Files(*.txt);;Rich Text Files(*.html *.htm);;All Files (*)"));
//     if (!fn.isEmpty() && maybeSave())
// 		{
//         load(fn);
// 
// 		filesTree->selectionModel()->reset();
// 		resultsList->selectionModel()->reset();
// 		textEdit->setFocus();
// 		}
// }