// Copyright 2008 Serkan Cabi
// This file is part of Kognite.
// 
// Kognite is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Kognite is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Kognite.  If not, see <http://www.gnu.org/licenses/>.

// Includes code from QT examples

#include "header.h"

void MainWindow::loadPdf(QString pdfFile)
{
	clearPdfCache();
	pdfScene->setBackgroundBrush(Qt::black);
	QImage pageImage;
	
	
	documentLoading = true;
	Poppler::Document *document = Poppler::Document::load(pdfFile);
	document->setRenderHint(Poppler::Document::TextAntialiasing);
	
	pdfViewer->setScene(pdfScene);
	
	int numPages = document->numPages();
	int pagePos = 5;
	int pdfSeparation = 10;
	pdfWidth = pdfViewer->width()-30;
	
	setWindowTitle(QFileInfo(pdfFile).fileName()+" ("+QString::number(numPages)+" pages)"); 
	
	for (int i = 0; i < numPages; i++) 
	{
		statusBar()->showMessage("Loading file: "+QFileInfo(pdfFile).fileName()+" (Page "+QString::number(i+1)+" of "+QString::number(numPages)+")");
		pdfResolution = pdfWidth * 72.0 / document->page(i)->pageSize().width();
		pageImage=document->page(i)->renderToImage(pdfResolution,pdfResolution);
		qApp->processEvents();
		if(pdfFile!=fileName) 
		{
			statusBar()->clearMessage();
			documentLoading = false;
			delete document;
			return;
		}
		pdfPageItems.append(new QGraphicsPixmapItem);
		qApp->processEvents();
		if(pdfFile!=fileName) 
		{
			statusBar()->clearMessage();
			documentLoading = false;
			delete document;
			return;
		}
		pdfPageItems.at(i)->setPixmap(QPixmap::fromImage(pageImage));
		qApp->processEvents();
		if(pdfFile!=fileName) 
		{
			statusBar()->clearMessage();
			documentLoading = false;
			delete document;
			return;
		}
		pdfPageItems.at(i)->setPos(QPointF(0, pagePos));
		pagePos += pdfPageItems.at(i)->pixmap().height() + pdfSeparation;
		qApp->processEvents();
		if(pdfFile!=fileName) 
		{
			statusBar()->clearMessage();
			documentLoading = false;
			delete document;
			return;
		}
		//QRectF paperRect = pdfScene->items()[i]->sceneBoundingRect();
		//QRectF paperRect = pdfPageItems.at(i)->sceneBoundingRect();
		//QPainterPath path;
	    //path.addRect(paperRect);
		//pdfScene->addPath(path);
		
		//page shadow
		/*		
		 qreal shWidth = paperRect.width()/100;
		 //QPixmap shadowPixmap;
		 
		 //right shadow
		 QPixmap rShadowPixmap = QPixmap(shWidth,paperRect.height()-shWidth);
		 QPainter rPainter(&rShadowPixmap);
		 QLinearGradient rgrad(rShadowPixmap.rect().topLeft(), rShadowPixmap.rect().topRight());
		 rgrad.setColorAt(0.0, QColor(0,0,0,255));
		 rgrad.setColorAt(1.0, QColor(0,0,0,0));
		 rPainter.fillRect(rShadowPixmap.rect(), QBrush(rgrad));
		 pdfScene->addPixmap(rShadowPixmap)->setOffset(paperRect.topRight()+ QPointF(0, shWidth));
		 
		 
		 //bottom shadow
		 QPixmap bShadowPixmap = QPixmap(paperRect.width()-shWidth,shWidth);
		 QPainter bPainter(&bShadowPixmap);
		 QLinearGradient bgrad(bShadowPixmap.rect().topLeft(), bShadowPixmap.rect().bottomLeft());
		 bgrad.setColorAt(0.0, QColor(0,0,0,255));
		 bgrad.setColorAt(1.0, QColor(0,0,0,0));
		 bPainter.fillRect(bShadowPixmap.rect(), QBrush(bgrad));
		 pdfScene->addPixmap(bShadowPixmap)->setOffset(paperRect.bottomLeft() + QPointF(shWidth, 0));
		 
		 
		 //corner shadow
		 QPixmap cShadowPixmap = QPixmap(shWidth,shWidth);
		 //QPixmap cShadowPixmap = QPixmap(15,15);
		 QPainter cPainter(&cShadowPixmap);
		 QRadialGradient cgrad(cShadowPixmap.rect().topLeft(), shWidth);
		 cgrad.setColorAt(0.0, QColor(0,0,0,255));
		 cgrad.setColorAt(1.0, QColor(0,0,0,0));
		 cPainter.fillRect(cShadowPixmap.rect(), QBrush(cgrad));
		 pdfScene->addPixmap(cShadowPixmap)->setOffset(paperRect.bottomRight() - QPointF(1, 1));
		 */
		pdfScene->addItem(pdfPageItems.at(i));
		qApp->processEvents();
		if(pdfFile!=fileName) 
		{
			statusBar()->clearMessage();
			documentLoading = false;
			delete document;
			return;
		}
	}
	statusBar()->clearMessage();
	documentLoading = false;
	delete document;
	
	actionZoomIn->setEnabled(true);
	actionZoomOut->setEnabled(true);
	tbZoomIn->setEnabled(true);
	tbZoomOut->setEnabled(true);
}

void MainWindow::pdfZoom(double pdfZoomFactor)
{
	
	actionZoomIn->setEnabled(false);
	actionZoomOut->setEnabled(false);
	tbZoomIn->setEnabled(false);
	tbZoomOut->setEnabled(false);
	
	QString pdfFile = fileName;
	pdfResolution *= pdfZoomFactor;

	pdfScene->setBackgroundBrush(Qt::black);
	QImage pageImage;
	
	documentLoading = true;
	Poppler::Document *document = Poppler::Document::load(pdfFile);
	document->setRenderHint(Poppler::Document::TextAntialiasing);
	
	int numPages = document->numPages();
	int newPagePos;
	int scrollPos = pdfViewer->verticalScrollBar()->value() * pdfZoomFactor;

	for (int i = 0; i < numPages; i++) 
	{
		statusBar()->showMessage("Resizing file: "+QFileInfo(pdfFile).fileName()+" (Page "+QString::number(i+1)+" of "+QString::number(numPages)+")");
		pageImage=document->page(i)->renderToImage(pdfResolution,pdfResolution);
		qApp->processEvents();
		if(pdfFile!=fileName) 
		{
			statusBar()->clearMessage();
			documentLoading = false;
			delete document;
			return;
		}
		pdfPageItems[i]->setPixmap(QPixmap::fromImage(pageImage));
		qApp->processEvents();
		if(pdfFile!=fileName) 
		{
			statusBar()->clearMessage();
			documentLoading = false;
			delete document;
			return;
		}
		newPagePos = pdfPageItems[i]->pos().y()*pdfZoomFactor;
		pdfPageItems[i]->setPos(QPointF(0, newPagePos));
		qApp->processEvents();
		if(pdfFile!=fileName) 
		{
			statusBar()->clearMessage();
			documentLoading = false;
			delete document;
			return;
		}
	}
	pdfScene->setSceneRect(pdfScene->itemsBoundingRect());
	pdfViewer->verticalScrollBar()->setValue(scrollPos);
	
	statusBar()->clearMessage();
	documentLoading = false;
	delete document;
	
	actionZoomIn->setEnabled(true);
	actionZoomOut->setEnabled(true);
	tbZoomIn->setEnabled(true);
	tbZoomOut->setEnabled(true);
}

void MainWindow::pdfZoomIn()
{
	pdfZoom(1.25);
}

void MainWindow::pdfZoomOut()
{
	pdfZoom(0.8);
}

// void MainWindow::pdfFind(QString searchString, bool forward, bool backward)
// {
//	//Make a global pdfDocumenet
// 	Poppler::Page* pdfPage = document->page(0);
// 	QString searchString="Fortran";
// 	QRectF resultLoc;
// 	
// 	pdfPage->search(searchString, resultLoc, Poppler::Page::FromTop, Poppler::Page::CaseInsensitive);
// 	resultLoc.setCoords (resultLoc.left()*resolution-2, resultLoc.top()*resolution-2, resultLoc.right()*resolution+2, resultLoc.bottom()*resolution+2);
// 	
// 	//Add the top left point of the page to the rect
// 	
// 	//Add rect
// }

void MainWindow::clearPdfCache()
{
	for (int i=0; i<pdfPageItems.count(); i++)
	{
		delete pdfPageItems[i];
	}
	pdfPageItems = QList<QGraphicsPixmapItem*>();
	delete pdfScene;
	pdfScene = new QGraphicsScene(pdfViewer);
}

bool MainWindow::pdfIndexer(QString file2IndexPath)
{
	try
	{
		QString content;
		QString file2IndexName = QFileInfo(file2IndexPath).fileName();
		
		if (!QFile::exists(file2IndexPath))
		{
			return false;
		}
		
		// Open the database for update, creating a new database if necessary.
		Xapian::WritableDatabase db(databasePath.toStdString().c_str(), Xapian::DB_CREATE_OR_OPEN);
		
		Xapian::TermGenerator indexer;
		Xapian::Stem stemmer("english");
		indexer.set_stemmer(stemmer);
		
		Poppler::Document *document = Poppler::Document::load(file2IndexPath);
		int numPages = document->numPages();
		
		Xapian::Document doc;
		doc.set_data( file2IndexPath.toStdString().c_str() );
		indexer.set_document(doc);
		
		indexer.index_text(file2IndexName.toStdString()); //index filename
		QList<Poppler::TextBox*> pageWordList;
		
		int i;
		int j;
		
		for (i = 0; i < numPages; i++) 
		{
			pageWordList = document->page(i)->textList();
			for(j = 0; j<pageWordList.count(); j++)
			{
				content = pageWordList[j]->text();
				//content = document->page(i)->textList().at(j)->text();
				//textEdit->append(document->page(i)->textList().at(j)->text());
				indexer.index_text(content.toStdString());
			}
		}
		
		// Add the document to the database
		db.add_document(doc);
		delete document;
		return true;
	}		catch(const Xapian::Error &error) 
	{
		string errorMessage = "Database error in fileIndexer: "+error.get_msg();
		statusBar()->showMessage(tr(errorMessage.c_str()),messageTime);
	}
	return false;
}