// Copyright 2008 Serkan Cabi
// This file is part of Kognite.
// 
// Kognite is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Kognite is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Kognite.  If not, see <http://www.gnu.org/licenses/>.

// Includes code from QT examples

#include "header.h"

MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
{
#ifdef Q_WS_MAC
	textEdit = new QtSpellCheckingTextEdit();
#else
	textEdit = new QTextEdit();
#endif
	
	settings = new QSettings("Kognite", "Kognite");
	
	cantOpenWidget = new QFrame();
	cantOpenWidget->hide();
	cantOpenLabel = new QLabel(tr("Kognite can not read this type of file.\n"),this);
	openExternallyButton = new QPushButton(tr("Open externally..."));
	connect(openExternallyButton, SIGNAL(clicked()), this, SLOT(openExternally()));
	
	pdfViewer = new QGraphicsView();
	pdfViewer->hide();
	pdfViewer->setInteractive(false);
	pdfViewer->setRenderHint(QPainter::Antialiasing, false);
	pdfViewer->setDragMode(QGraphicsView::ScrollHandDrag);
	pdfViewer->setViewportUpdateMode(QGraphicsView::SmartViewportUpdate);
	pdfScene = new QGraphicsScene(pdfViewer);
	
	defaultTextFrameStyle = textEdit->frameStyle();
	defaultPdfFrameStyle = pdfViewer->frameStyle();
	initialLaunch = true;
	inFullScreen = false;
	searchResults = false;
	documentLoading = false;
	messageTime = 10000; //seconds*1000, set to zero for unlimited time
	nameColumnWidth = 250;
	file2Rename = "";
	dir2Rename = "";
	forcedExtension1 = ".txt";
	forcedExtension2 = " ";
	
	statusBar()->setFixedHeight(20);
	statusBar()->setContentsMargins(0,0,0,0);
	statusBar()->showMessage(tr("Ready"),messageTime);
	
	searchBox = new QLineEdit();
	connect(searchBox,SIGNAL(returnPressed()), this, SLOT(searcher()));
	
	findFrame = new QFrame();
	findLayout = new QGridLayout();
	findBox = new QLineEdit();
	findBar = new QToolBar();
	replaceBox = new QLineEdit();
	QToolButton *findClose = new QToolButton(this);
	QPushButton *nextButton = new QPushButton(tr("Next"));
	QPushButton *prevButton = new QPushButton(tr("Previous"));
	replaceWithButton = new QPushButton(tr("Replace With..."));
	replaceButton = new QPushButton(tr("Replace"));
	replaceAllButton = new QPushButton(tr("Replace All"));
	checkCase = new QCheckBox(this);
	checkCase->setText("Case Sensitive ");
	checkCase->setChecked(false);
	checkWholeWords = new QCheckBox(this);
	checkWholeWords->setText("Whole Words");
	
	connect(findClose, SIGNAL(clicked()), this, SLOT(hideFindBox()));
	connect(findBox, SIGNAL(textChanged(const QString&)), this, SLOT(find(QString)));
	connect(findBox, SIGNAL(returnPressed()), nextButton, SLOT(click()));
	connect(nextButton, SIGNAL(clicked()), this, SLOT(findNext()));
	connect(prevButton, SIGNAL(clicked()), this, SLOT(findPrev()));
	connect(checkCase, SIGNAL(stateChanged(int)), this, SLOT(findWithNewOptions()));
	connect(checkWholeWords, SIGNAL(stateChanged(int)), this, SLOT(findWithNewOptions()));
	connect(replaceBox, SIGNAL(returnPressed()), replaceButton, SLOT(click()));
	connect(replaceWithButton, SIGNAL(clicked()), this, SLOT(showReplaceRow()));
	connect(replaceButton, SIGNAL(clicked()), this, SLOT(replaceWith()));
	connect(replaceAllButton, SIGNAL(clicked()), this, SLOT(replaceAll()));
	
	findFrame->setContentsMargins(0,0,0,0);
	findLayout->setContentsMargins(0,0,0,0);
	findBar->setContentsMargins(0,0,0,0);
	findBar->addWidget(findClose);
	
	findBox->setFixedHeight(22);
	replaceBox->setFixedHeight(22);
	findLayout->addWidget(findBar,0,0,1,1);
	findLayout->addWidget(findBox,0,1,1,1);
	findLayout->addWidget(prevButton,0,2,1,1);
	findLayout->addWidget(nextButton,0,3,1,1);
	findLayout->addWidget(checkCase,0,4,1,1);
	findLayout->addWidget(checkWholeWords,0,5,1,1);
	findLayout->addWidget(replaceWithButton,0,6,1,1);
	findLayout->addWidget(replaceBox,1,1,1,1);
	findLayout->addWidget(replaceButton,1,2,1,1);
	findLayout->addWidget(replaceAllButton,1,3,1,1);
	
	findLayout->setColumnStretch(1,2);
	findLayout->setColumnStretch(7,1);
	findLayout->setColumnMinimumWidth(6,120);
	
	prevButton->setFixedWidth(100);
	nextButton->setFixedWidth(100);
	replaceButton->setFixedWidth(100);
	replaceAllButton->setFixedWidth(100);
	replaceButton->setDisabled(true);
	
	findClose->setIcon(QIcon(":/Images/close.png"));
	findClose->setFixedSize(22,22);
	findClose->setAutoRaise(false);
	
	findFrame->setFrameStyle(QFrame::StyledPanel);
	findFrame->setLayout(findLayout);
	findFrame->hide();
	
	textEdit->setAutoFormatting(QTextEdit::AutoNone);
	textEdit->setTabStopWidth(32);
	connect(textEdit, SIGNAL(currentCharFormatChanged(const QTextCharFormat &)), this, SLOT(currentCharFormatChanged(const QTextCharFormat &)));
    connect(textEdit, SIGNAL(cursorPositionChanged()), this, SLOT(cursorPositionChanged()));
	connect(textEdit, SIGNAL(selectionChanged()), this, SLOT(disableReplace()));
	connect(textEdit, SIGNAL(textChanged()), this, SLOT(processTextChange()));
	connect(textEdit->document(), SIGNAL(modificationChanged(bool)), this, SLOT(setWindowModified(bool)));
	
	QPalette p = palette();
	p.setColor(QPalette::Inactive, QPalette::Highlight,
			   p.color(QPalette::Active, QPalette::Highlight));
	p.setColor(QPalette::Inactive, QPalette::HighlightedText,
			   p.color(QPalette::Active, QPalette::HighlightedText));
	textEdit->setPalette(p);
	
	setupMenuActions();
	prepareToolBar();
	
    setWindowModified(textEdit->document()->isModified());
	
	actionSave->setEnabled(false);
    actionUndo->setEnabled(false);
    actionRedo->setEnabled(false);
    actionCopy->setEnabled(false);
    actionCut->setEnabled(false);
    actionCrop->setEnabled(false);
	tbZoomIn->setEnabled(false);
	tbZoomOut->setEnabled(false);
	
	searchContainer = new QGroupBox(this);
	
	resultsList = new QListWidget(this);
	connect(resultsList, SIGNAL(itemSelectionChanged()), this, SLOT(loadResult()));
	
	similarsButton = new QPushButton(tr("Similars"));
	connect(similarsButton, SIGNAL(clicked()), this, SLOT(getSimilar()));
	similarsButton->setFixedWidth(90);
	loadConfigFile();
	
	filesTree = new MyTreeView;
	filesModel = new QDirModel(QStringList(),QDir::Files,QDir::Name);
	filesModel->setReadOnly ( false );
	filesTree->setEditTriggers(QAbstractItemView::NoEditTriggers);
	filesTree->setModel(filesModel);
	filesTree->setRootIndex(filesModel->index(filesPath));
	filesTree->setColumnWidth(0,nameColumnWidth);
	filesTree->resizeColumnToContents(1);
	filesTree->resizeColumnToContents(2);
	filesTree->resizeColumnToContents(3);
	filesTree->setDragDropMode(QAbstractItemView::DragOnly);
	connect(filesTree->selectionModel(), SIGNAL(selectionChanged(QItemSelection, QItemSelection)), this, SLOT(loadClicked()));
	connect(filesTree, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(renameFileStart(QModelIndex)));
	connect(filesTree->itemDelegate(), SIGNAL(closeEditor(QWidget *)), this, SLOT(renameFileEnd(QWidget *)));
	
	dirsTree = new MyTreeView;
	dirsModel = new QDirModel(QStringList(), QDir::Dirs | QDir::NoDotAndDotDot, QDir::Name);
	dirsTree->setEditTriggers(QAbstractItemView::NoEditTriggers);
	dirsTree->setModel(dirsModel);
	prepareDirsTree();
	connect(dirsTree, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(renameDirStart(QModelIndex)));
	connect(dirsTree->itemDelegate(), SIGNAL(closeEditor(QWidget *)), this, SLOT(renameDirEnd(QWidget *)));
	connect(dirsTree->selectionModel(), SIGNAL(selectionChanged(QItemSelection, QItemSelection)), this, SLOT(loadDirectory()));
	connect(dirsTree, SIGNAL(fileMoveRequested(QString, QPoint)), this, SLOT(moveFile(QString, QPoint)));
	connect(dirsTree, SIGNAL(directoryMoveRequested(QString, QPoint)), this, SLOT(moveDirectory(QString, QPoint)));
	
	// QFont font;
	// font = dirsTree->font();
	// font.setPointSize(11);
	// dirsTree->setFont(font);
	// filesTree->setFont(font);
	// dirsTree->setIconSize(QSize(15,15));
	// filesTree->setIconSize(QSize(15,15));
	
	prepareLayout();
	
	setTabOrder(searchBox,resultsList);
	setTabOrder(resultsList,findBox);
	setTabOrder(findBox,replaceBox);
	setTabOrder(replaceBox,dirsTree);
	setTabOrder(dirsTree,filesTree);
	setTabOrder(filesTree,textEdit);
	
	setRichTextActions(false);
	filesTree->setFocus();
	initialLaunch = false;
}

void MainWindow::processTextChange()
{
	findBox->setPalette(QApplication::palette());
	if(statusBar()->currentMessage() == "Not found" || statusBar()->currentMessage().endsWith("matches"))
		statusBar()->clearMessage();
}

void MainWindow::setCurrentFileName(const QString &fileName)
{
    this->fileName = fileName;
    textEdit->document()->setModified(false);
	
    QString shownName;
    if (fileName.isEmpty())
        shownName = "";
    else
        shownName = QFileInfo(fileName).fileName();
	
    //setWindowTitle(tr("%1[*] - %2").arg(shownName).arg(tr("Plain Text")));
	setWindowTitle(shownName);
    setWindowModified(false);
}

bool MainWindow::loadConfigFile()
{
	bool shouldIndex = false;
	if(!settings->contains("filesPath")) 
	{
		QMessageBox::information(this, tr("Welcome"),
								 tr("Welcome to Kognite.\nKognite is a program to organize your text documents. On the next screen you will select where your documents are stored."), QMessageBox::Ok);
		chooseFilesPath(false);
		shouldIndex = true;
	}
    if(!settings->contains("databasePath")) 
	{
		QMessageBox::information(this, tr("Welcome"),
								 tr("Kognite indexes your documents for search functionality. On the next screen you will select where your database is stored.\nThis directory must be empty and you should not add or remove any files in the future. Also it can not be in your files directory."), QMessageBox::Ok);
		chooseDbPath(false);
		shouldIndex = true;
	}
	
	filesPath = settings->value("filesPath").toString();
	databasePath = settings->value("databasePath").toString();
	
	if (!QDir(filesPath).exists() || !QDir(databasePath).exists() || !filesPath.endsWith('/') || !databasePath.endsWith('/')) 
	{
		//Make this message a window
		chooseFilesPath(false);
		chooseDbPath(false);
		shouldIndex = true;
		statusBar()->showMessage(tr("Configuration file was invalid."),messageTime);
	}
	if (shouldIndex) indexer();
	return true;
}

void MainWindow::updateConfigFile()
{
	settings->setValue("filesPath", filesPath);
	settings->setValue("databasePath", databasePath);
}

bool MainWindow::load(const QString &f)
{
	bool spellCheckToggled = false;
	hideFindBox();
	actionZoomIn->setEnabled(false);
	actionZoomOut->setEnabled(false);
	tbZoomIn->setEnabled(false);
	tbZoomOut->setEnabled(false);
	actionPrint->setEnabled(false);
	actionFind->setEnabled(false);
	actionFindNext->setEnabled(false);
	actionFindPrev->setEnabled(false);
	actionFindAndReplace->setEnabled(false);
	tbFind->setEnabled(false);
	
    if (!QFile::exists(f))
        return false;
    QFile file(f);
    if (!file.open(QFile::ReadOnly))
        return false;
	
	documentLoading = true;
#ifdef Q_WS_MAC
	if (textEdit->asYouTypeAction->isChecked())
	{
		textEdit->toggleSyntaxHighlighting();
		spellCheckToggled = true;
	}
#endif
	setCurrentFileName(f);
	
	if (f.endsWith(".txt") || f.endsWith(".htm") || f.endsWith(".html"))
	{
		textEdit->show();
		cantOpenWidget->hide();
		pdfViewer->hide();
		clearPdfCache();
		actionFullScreen->setEnabled(true);
		tbFulScreen->setEnabled(true);
		actionPrint->setEnabled(true);
		actionFind->setEnabled(true);
		actionFindNext->setEnabled(true);
		actionFindPrev->setEnabled(true);
		actionFindAndReplace->setEnabled(true);
		tbFind->setEnabled(true);
		
		QByteArray data = file.readAll();
		QTextCodec *codec = Qt::codecForHtml(data);
		QString str = codec->toUnicode(data);
		if (f.endsWith(".htm") || f.endsWith(".html")) 
		{
			setRichTextActions(true);
			textEdit->setHtml(str);
			forcedExtension1 = ".html";
			forcedExtension2 = ".htm";
		} else 
		{
			setRichTextActions(false);
			textEdit->setCurrentCharFormat(QTextCharFormat());
			str = QString::fromLocal8Bit(data);
			textEdit->setPlainText(str);
			forcedExtension1 = ".txt";
		}
	}
	else if(f.endsWith(".pdf"))
	{	
		pdfViewer->show();
		textEdit->hide();
		cantOpenWidget->hide();
		setRichTextActions(false);
		actionFullScreen->setEnabled(true);
		tbFulScreen->setEnabled(true);
		forcedExtension1 = ".pdf";
		loadPdf(f);	
	}
	else
	{
		textEdit->hide();
		pdfViewer->hide();
		clearPdfCache();
		cantOpenWidget->show();
		setRichTextActions(false);
		actionFullScreen->setEnabled(false);
		tbFulScreen->setEnabled(false);
		forcedExtension1 = " ";
	}
	
#ifdef Q_WS_MAC
	if (spellCheckToggled)
	{
		textEdit->toggleSyntaxHighlighting();
	}
#endif
	documentLoading = false;
    return true;
}

void MainWindow::loadClicked()
{
	//Set selection of the results view to null
	//resultsList->setFocus();
	resultsList->selectionModel()->reset();
	filesTree->setFocus();
	
	//Show selected file
	if(filesTree->selectionModel()->selectedIndexes().count())
	{
		if(maybeSave())
		{
			QModelIndex qm = filesTree->selectionModel()->selectedIndexes()[0];
			load(filesModel->filePath(qm));
		}
	}
}

void MainWindow::loadDirectory()
{
	if(dirsTree->selectionModel()->selectedIndexes().count())
	{
		filesTree->selectionModel()->reset();
		currentDirPath = dirsModel->filePath(dirsTree->selectionModel()->selectedIndexes()[0]);
		if (currentDirPath.at(currentDirPath.length()-1)!='/') currentDirPath.append("/");
		filesModel->setFilter(QDir::Files);
		filesTree->setModel(filesModel);
		filesTree->setRootIndex(filesModel->index(currentDirPath));
		filesTree->scrollToTop();
	}	
}

void MainWindow::loadResult()
{
	//Set selection of the directory view to null
	filesTree->setFocus();
	filesTree->selectionModel()->reset();
	
	//Show selected file
	if(resultsList->selectedItems().count()==1) //crashes without this if !!!
	{
		if(maybeSave())
		{
			QString resultFile=resultsList->selectedItems()[0]->toolTip();
			//resultFile.prepend(filesPath);
			resultsList->setFocus();
			load(resultFile);
		}
		if(searchResults)
		{
			checkCase->setChecked(false);
			checkWholeWords->setChecked(false);
			showFindBox();
			findBox->setText(""); //doesn't search again if you don't CHANGE the text
			findBox->setText(searchBox->text());
			replaceButton->setEnabled(true);
		}
		resultsList->setFocus();
	}
}

void MainWindow::openExternally()
{
	if (QFileInfo(fileName).exists())
	{
		QString prefix = "file://";
		QDesktopServices::openUrl(QUrl(prefix+fileName));
	}
}

void MainWindow::closeEvent(QCloseEvent *e)
{
    if (maybeSave())
        e->accept();
    else
        e->ignore();
}

void MainWindow::showFindBox()
{
	if(textEdit->isVisible())
	{
		replaceWithButton->show();
		replaceBox->hide();
		replaceButton->hide();
		replaceAllButton->hide();
		
		findFrame->show();
		findBox->setFocus();
		findBox->selectAll();
	}
}

void MainWindow::hideFindBox()
{
	if (findBox->hasFocus()) textEdit->setFocus();
	findFrame->hide();
	
	findBox->setPalette(QApplication::palette());
	
	statusBar()->clearMessage();
}

void MainWindow::showReplaceRow()
{
	replaceWithButton->hide();
	replaceBox->show();
	replaceButton->show();
	replaceAllButton->show();
	
	replaceBox->setFocus();
	replaceBox->selectAll();
}

void MainWindow::showFindAndReplace()
{
	showFindBox();
	showReplaceRow();
	findBox->setFocus();
	findBox->selectAll();
}

void MainWindow::keyPressEvent(QKeyEvent *e)
{
	int key = e->key();
	
	if (findBox->hasFocus() && key==Qt::Key_Escape) 
		hideFindBox();
	
	else if (replaceBox->hasFocus() && key==Qt::Key_Escape) 
		hideFindBox();
	
	else if(inFullScreen && key==Qt::Key_Escape)
		toggleFullScreen();
	
	else if(filesTree->hasFocus() && filesTree->selectionModel()->selectedIndexes().count() && (key==Qt::Key_Backspace || key==Qt::Key_Delete))
		deleteCurrentFile();
	
	else if(dirsTree->hasFocus() && dirsTree->selectionModel()->selectedIndexes().count() && (key==Qt::Key_Backspace || key==Qt::Key_Delete))
		dirDelete();
	
	else QWidget::keyPressEvent(e);
}

void MainWindow::findNext()
{
	findBox->setFocus();
	find(findBox->text(), true, false);
	replaceButton->setEnabled(true);
}

void MainWindow::findPrev()
{
	findBox->setFocus();
	find(findBox->text(), false, true);
	replaceButton->setEnabled(true);	
}

void MainWindow::findWithNewOptions()
{
	QString tempString = findBox->text();
	findBox->setText("");
	findBox->setText(tempString);
}

bool MainWindow::find(QString ttf, bool forward, bool backward)
{
	QString oldText = findBox->text();
	QTextCursor c = textEdit->textCursor();
	QTextDocument::FindFlags options;
	QPalette p = QApplication::palette();
	findBox->setPalette(p);
	statusBar()->clearMessage();
	
	if (c.hasSelection())
		c.setPosition(forward ? c.position() : c.anchor(), QTextCursor::MoveAnchor);
	
	QTextCursor newCursor = c;
	
	if (!ttf.isEmpty()) {
		
		if (backward)
			options |= QTextDocument::FindBackward;
		
		if (checkCase->isChecked())
			options |= QTextDocument::FindCaseSensitively;
		
		if (checkWholeWords->isChecked())
			options |= QTextDocument::FindWholeWords;
		
		newCursor = textEdit->document()->find(ttf, c, options);
		//labelWrapped->hide();
		
		if (newCursor.isNull()) 
		{
			QTextCursor ac(textEdit->document());
			ac.movePosition(options & QTextDocument::FindBackward
							? QTextCursor::End : QTextCursor::Start);
			newCursor = textEdit->document()->find(ttf, ac, options);
			if (newCursor.isNull()) 
			{
				p.setColor(QPalette::Active, QPalette::Base, QColor(255, 102, 102));
				newCursor = c;
			} 
			//else
			//labelWrapped->show();
		}
		
		QTextCursor counterCursor = c;
		int matchCount = -1;
		counterCursor.movePosition(options & QTextDocument::FindBackward
								   ? QTextCursor::End : QTextCursor::Start);
		while (!counterCursor.isNull())
		{
			counterCursor = textEdit->document()->find(ttf, counterCursor, options);
			matchCount++;
		}
		
		//findFrame->show();
		actionFindNext->setEnabled(true);
		actionFindPrev->setEnabled(true);
		textEdit->setTextCursor(newCursor);
		findBox->setPalette(p);
		
		if (matchCount)
		{
			statusBar()->showMessage(QString::number(matchCount)+tr(" matches"));
			replaceButton->setEnabled(true);
			return true;
		}
		else
		{
			statusBar()->showMessage(tr("Not found"));
			replaceButton->setDisabled(true);
			return false;
		}
	}
	return false;
}

void MainWindow::replaceWith()
{
	textEdit->textCursor().insertText(replaceBox->text());
}

void MainWindow::replaceAll()
{
	int i=0;
	textEdit->textCursor().beginEditBlock();
	while(find(findBox->text(), true, false))
	{
		replaceWith();
		i++;
	}
	textEdit->textCursor().endEditBlock();
	if(i>0)
	{
		statusBar()->showMessage(QString::number(i)+tr(" occurrences replaced."),messageTime);
	}
}

void MainWindow::disableReplace()
{
	replaceButton->setDisabled(true);
}

void MainWindow::refreshFilesTree()
{	
	int selected = filesTree->selectionModel()->selectedIndexes().count();
	//setUpdatesEnabled(false);
	filesModel->refresh();
	filesTree->setRootIndex(filesModel->index(currentDirPath));
	qApp->processEvents();
	if(selected)
	{
		filesTree->selectionModel()->select(filesModel->index(fileName),QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows);
		filesTree->scrollTo(filesModel->index(fileName));
	}
	//setUpdatesEnabled(true);
}

void MainWindow::refreshDirsTree()
{
	dirsModel->refresh();
	//dirsTree->selectionModel()->select(dirsModel->index(currentDirPath),QItemSelectionModel::ClearAndSelect);
}

void MainWindow::prepareToolBar()
{
	tb = new QToolBar();
	
	a = new QAction(QIcon(":/Images/home.png"), tr("Home Directory"), this);
	connect(a, SIGNAL(triggered()), this, SLOT(goHome()));
	tb->addAction(a);
	
	
	a = new QAction(QIcon(":/Images/new_folder.png"), tr("New Directory"), this);
	connect(a, SIGNAL(triggered()), this, SLOT(dirAdd()));
	tb->addAction(a);
	
	a = new QAction(QIcon(":/Images/new_plain_text.png"), tr("New Plain Text"), this);
	connect(a, SIGNAL(triggered()), this, SLOT(newPlainFile()));
	tb->addAction(a);
	
	a = new QAction(QIcon(":/Images/new_rich_text.png"), tr("New Rich Text"), this);
	connect(a, SIGNAL(triggered()), this, SLOT(newRichFile()));
	tb->addAction(a);
	
	a = new QAction(QIcon(":/Images/save.png"), tr("Save"), this);
	connect(a, SIGNAL(triggered()), this, SLOT(fileSave()));
	connect(textEdit->document(), SIGNAL(modificationChanged(bool)), a, SLOT(setEnabled(bool)));
	a -> setEnabled(false);
	tb->addAction(a);
	
	a = tbFind = new QAction(QIcon(":/Images/find.png"), tr("Find..."), this);
	connect(a, SIGNAL(triggered()), this, SLOT(showFindBox()));
	tb->addAction(a);
	
	a = tbZoomIn = new QAction(QIcon(":/Images/zoom_in.png"), tr("Zoom In"), this);
	connect(a, SIGNAL(triggered()), this, SLOT(pdfZoomIn()));
	tb->addAction(a);
	
	a = tbZoomOut = new QAction(QIcon(":/Images/zoom_out.png"), tr("Zoom Out"), this);
	connect(a, SIGNAL(triggered()), this, SLOT(pdfZoomOut()));
	tb->addAction(a);
	
	a = tbFulScreen = new QAction(QIcon(":/Images/fullscreen.png"), tr("Full Screen"), this);
	connect(a, SIGNAL(triggered()), this, SLOT(toggleFullScreen()));
	tb->addAction(a);
	
	a = new QAction(QIcon(":/Images/sync.png"), tr("Index Files"), this);
	connect(a, SIGNAL(triggered()), this, SLOT(indexer()));
	tb->addAction(a);
	
	addToolBar(tb);
	
	setUnifiedTitleAndToolBarOnMac(true);
	tb->setVisible(true);
	actionToolBar->setChecked(true);
}

void MainWindow::prepareDirsTree()
{
	refreshDirsTree();
	dirsModel->setReadOnly ( false );
	currentDirPath = filesPath;
	parentDirIndex = dirsModel->parent(dirsModel->index(filesPath));
	dirsTree->setRootIndex(parentDirIndex);
	dirsTree->setCurrentIndex(dirsModel->index(currentDirPath));
	dirsTree->expandToDepth(0);
	//dirsTree->expandAll();
	dirsTree->setDragDropMode(QAbstractItemView::DragDrop);
	dirsTree->hideColumn(1);
	dirsTree->hideColumn(2);
	dirsTree->hideColumn(3);
	dirsTree->setIndentation(10);
	
	for (int i=0;i<dirsModel->rowCount(parentDirIndex);i++)
	{
		if(dirsModel->index(filesPath).row()!=i)
		{	
			dirsTree->setRowHidden(i,parentDirIndex,true);
		}
		else
		{
			dirsTree->setRowHidden(i,parentDirIndex,false);
		}
	}
	
}

void MainWindow::prepareLayout()
{
	DocumentViewer = new QFrame(this);
	DocumentViewerLayout = new QGridLayout(this);
	DocumentViewerLayout->setContentsMargins(0,0,0,0);
	DocumentViewerLayout->setSpacing(0);
	DocumentViewerLayout->addWidget(textEdit,0,1,1,1);
	DocumentViewerLayout->addWidget(findFrame,1,1,1,1);
	DocumentViewerLayout->addWidget(cantOpenWidget,0,1,1,1);
	DocumentViewerLayout->addWidget(pdfViewer,0,1,1,1);
	
	cantOpenLayout = new QVBoxLayout;
	cantOpenLayout->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
	cantOpenLayout->addWidget(cantOpenLabel);
	cantOpenLayout->addWidget(openExternallyButton);
	cantOpenWidget->setLayout(cantOpenLayout);
	
	middleFrame = new QSplitter(this);
	middleFrame->setHandleWidth(5);
	middleFrame->setOrientation(Qt::Vertical);
	middleFrame->addWidget(filesTree);
	middleFrame->addWidget(DocumentViewer);
	
	DocumentViewerLayout->setColumnStretch(0,0);
	DocumentViewerLayout->setColumnStretch(1,1);
	DocumentViewerLayout->setColumnStretch(2,0);
	middleFrame->setContentsMargins(0,5,0,0);
	
	DocumentViewer->setLayout(DocumentViewerLayout);
	
	if(initialLaunch)
	{
		
		leftFrame = new QFrame(this);
		leftLayout = new QGridLayout(this);
		leftLayout->setContentsMargins(0,5,0,0);
		leftLayout->setSpacing(0);
		leftLayout->addWidget(dirsTree,0,0,1,1);
		leftFrame->setLayout(leftLayout);
		
		middleFrame->setStretchFactor(0,1);
		middleFrame->setStretchFactor(1,2);
		
		searchLayout = new QGridLayout;
		searchLayout->setContentsMargins(0,0,0,0);
		searchLayout->setSpacing(0);
		searchLayout->addWidget(searchBox,0,0,1,1);
		searchContainer->setLayout(searchLayout);
		
		rightBar = new QToolBar();
		//rightBar->setLayoutDirection(Qt::RightToLeft);
		rightBar->setFixedHeight(30);
		rightBar->addWidget(similarsButton);
		
		rightFrame = new QFrame(this);
		rightLayout = new QGridLayout(this);
		rightLayout->setContentsMargins(0,5,0,0);
		rightLayout->setSpacing(5);
		rightLayout->addWidget(searchContainer,0,0,1,1);
		rightLayout->addWidget(resultsList,1,0,1,1);
		rightLayout->addWidget(rightBar,2,0,1,1);
		rightFrame->setLayout(rightLayout);
	}
	
	central = new QSplitter(this);
	central->setContentsMargins(5,0,5,0);
	central->setHandleWidth(5);
	central->addWidget(leftFrame);
	central->addWidget(middleFrame);
	central->addWidget(rightFrame);
	central->setStretchFactor(0,2);
	central->setStretchFactor(1,7);
	central->setStretchFactor(2,2);
	setCentralWidget(central);
}

void MainWindow::moveDirectory(QString directory, QPoint droppedPoint)
{
	QString destination = dirsModel->filePath(dirsTree->indexAt(droppedPoint));
	QString directoryName = directory;
	directoryName = directoryName.remove(0,directoryName.lastIndexOf("/"));
	destination.append(directoryName);
	QDir dirObject;
	if (dirObject.exists(destination))
	{
		directoryName.remove(0,1);
		QMessageBox::information(this, "Drop Event", "A directory called \""+directoryName+"\" already exists in this location.", QMessageBox::Ok);
		return;
	}
	
	if (destination.startsWith(directory))
	{
		QMessageBox::information(this, "Drop Event", "Can not move a directory to its own subdirectory.", QMessageBox::Ok);
		return;
	}
	
	if(!dirObject.rename(directory, destination))
	{
		QMessageBox::information(this, "Drop Event", "Directory could not be moved!", QMessageBox::Ok);
		return;
	}
	renameDirInDB(directory,destination);
	currentDirPath = destination;
	refreshAll();
}

void MainWindow::moveFile(QString file, QPoint droppedPoint)
{
	QString destination = dirsModel->filePath(dirsTree->indexAt(droppedPoint));
	QString filePath = file;
	filePath = filePath.remove(0,fileName.lastIndexOf("/"));
	destination.append(filePath);
	QDir dirObject;
	
	if (dirObject.exists(destination))
	{
		filePath.remove(0,1);
		QMessageBox::information(this, "Drop Event", "A file called \""+filePath+"\" already exists in this location.", QMessageBox::Ok);
		return;
	}
	
	if(!dirObject.rename(file, destination))
	{
		QMessageBox::information(this, "Drop Event", "File could not be moved!", QMessageBox::Ok);
	}
	renameInDB(file, destination);
	refreshAll();
	filesTree->setFocus();
}

void MainWindow::deleteCurrentFile()
{
	QDir dirObject;
	
	if(dirObject.exists(fileName))
	{
		QMessageBox msgBox;
		msgBox.setWindowTitle(tr("Delete Confirmation"));
		msgBox.setText(tr("Current file will be deleted.\nThis action can not be undone."));
		QPushButton *deleteButton = msgBox.addButton(tr("Delete"), QMessageBox::ActionRole);
		msgBox.addButton(QMessageBox::Cancel);
		
		msgBox.exec();
		
		if (msgBox.clickedButton() == deleteButton) 
		{
			if(dirObject.remove(fileName))
			{
				removeFromDB(fileName);
				refreshAll();
				textEdit->clear();
				setCurrentFileName(QString());
				statusBar()->showMessage(tr("File is deleted."));
			}
			else
			{
				statusBar()->showMessage(tr("File could not be deleted."));	
			}
		}
	}
	return;
}

void MainWindow::renameFileStart(QModelIndex index)
{
	if(!documentLoading)
	{
		file2Rename = filesModel->filePath(index);
		filesTree->setCurrentIndex(index);
		filesTree->edit(index);
	}
}

void MainWindow::renameFileEnd(QWidget * editor)
{
	editor->close(); //Just to get rid of the warning
	qApp->processEvents();
	
	if(file2Rename != "")
	{
		QString newFilePath = filesModel->filePath(filesTree->selectionModel()->selectedIndexes()[0]);
		renameInDB(file2Rename, newFilePath);
		if(forcedExtension1 != " " && !newFilePath.endsWith(forcedExtension1) && !newFilePath.endsWith(forcedExtension2))
		{
			QDir dirObject;
			QString updatedPath = newFilePath;
			while(dirObject.exists(updatedPath+forcedExtension1))
			{
				updatedPath.append("_");
			}			
			updatedPath.append(forcedExtension1);
			dirObject.rename(newFilePath, updatedPath);
			//renameInDB(newFilePath, updatedPath);
			removeFromDB(newFilePath);
			fileIndexer(updatedPath);
			setCurrentFileName(updatedPath);
			
			
			//Ugly workaround (QT bug?)
			setUpdatesEnabled(false);
			refreshAll();
			qApp->processEvents();
			refreshAll();
			filesTree->setCurrentIndex(filesModel->index(fileName));
			filesTree-> selectionModel()-> select(filesModel->index(fileName),QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows);
			setUpdatesEnabled(true);
		}
		file2Rename = "";
	}
}

void MainWindow::renameDirStart(QModelIndex index)
{
	if(!documentLoading)
	{	
		dir2Rename = dirsModel->filePath(index);
		dirsTree->setCurrentIndex(index);
		dirsTree->edit(index);
	}
}

void MainWindow::renameDirEnd(QWidget * editor)
{
	editor->size(); //Just to get rid of the warning
	if(dir2Rename != "")
	{
		renameDirInDB(dir2Rename, dirsModel->filePath(dirsTree->selectionModel()->selectedIndexes()[0]));
		dir2Rename = "";
	}
}

void MainWindow::dirAdd()
{
	QDir dirObject;
	QString dirName = currentDirPath+"Untitled";
	
	if(!dirsTree->selectionModel()->selectedIndexes().count())
		goHome();
	
	while(dirObject.exists(dirName))
	{
		dirName.append("_");
	}
	dirsModel->mkdir(dirsTree->selectionModel()->selectedIndexes()[0],dirName);
	dirsTree->expand(dirsModel->index(currentDirPath));
	dirsTree->setCurrentIndex(dirsModel->index(dirName));
	dirsTree->edit(dirsModel->index(dirName));
	dirsTree->selectionModel()->select(dirsModel->index(dirName),QItemSelectionModel::ClearAndSelect);
	
}

void MainWindow::dirDelete()
{
	if(dirsTree->selectionModel()->selectedIndexes().count())
	{
		QMessageBox msgBox;
		msgBox.setWindowTitle(tr("Delete Confirmation"));
		msgBox.setText(tr("Current directory will be deleted with all of its contents.\nThis action can not be undone."));
		QPushButton *deleteButton = msgBox.addButton(tr("Delete"), QMessageBox::ActionRole);
		msgBox.addButton(QMessageBox::Cancel);
		
		msgBox.exec();
		
		QString path = dirsModel->filePath(dirsTree->selectionModel()->selectedIndexes()[0]);
		
		if (msgBox.clickedButton() == deleteButton) 
		{
			QString parentPath = dirsModel->filePath( dirsTree->selectionModel()->selectedIndexes()[0].parent());
			QDir dir2Delete = QDir(dirsModel->filePath(dirsTree->selectionModel()->selectedIndexes()[0]));
			if(dirDeleter(dir2Delete))
			{
				refreshAll();
				dirsTree->selectionModel()->select(dirsModel->index(parentPath),QItemSelectionModel::ClearAndSelect);
				removeDirFromDB(path);
				statusBar()->showMessage(tr("Directory is deleted."),messageTime);
			}
			else
			{
				statusBar()->showMessage(tr("Directory could not be deleted."),messageTime);	
			}
		}
	}
	return;
}

bool MainWindow::dirDeleter(QDir dir2Delete)
{
	QStringList list;
	int dirCount = dir2Delete.entryList(QDir::NoDotAndDotDot | QDir::Dirs).count();
	list = dir2Delete.entryList(QDir::NoDotAndDotDot | QDir::Dirs);
	for(int i=0;i<dirCount;i++)
	{
		if(!dirDeleter(QDir(dir2Delete.filePath(list[i]))))
		{
			return false;
		}
	}
	
	list =  dir2Delete.entryList(QDir::Files);
	int fileCount = list.count();
	QDir dirObject;
	for(int i=0;i<fileCount;i++)
	{
		if(!dirObject.remove(dir2Delete.filePath(list[i]))) return false;
	}
	
	return dirsModel->rmdir(dirsModel->index(dir2Delete.path()));
}

void MainWindow::goHome()
{
	dirsTree->selectionModel()->select(dirsModel->index(filesPath),QItemSelectionModel::ClearAndSelect);
}

void MyTreeView::dropEvent(QDropEvent *event)
{
	setFocus();
	QDirModel dirStructure;
	if (dropIndicatorPosition() == QAbstractItemView::OnItem)
	{
		
		if (dirStructure.isDir(dirStructure.index(event->mimeData()->urls()[0].path())))
		{
			emit directoryMoveRequested(event->mimeData()->urls()[0].path(), event->pos());
			setState(QAbstractItemView::NoState);
			event->accept();
			return;
		}
		
		else
		{
			emit fileMoveRequested(event->mimeData()->urls()[0].path(), event->pos());
			setState(QAbstractItemView::NoState);
			event->accept();
			return;
		}
		
	}
	else
	{
		setState(QAbstractItemView::NoState);
		event->ignore();
	}
}
